@@ -30,12 +30,13 @@
 const nodeConstants = require("resource://devtools/shared/dom-node-constants.js");
 const {
   getBindingElementAndPseudo,
-  getCSSStyleRules,
+  getMatchingCSSRules,
   hasVisitedState,
   isAgentStylesheet,
   isAuthorStylesheet,
   isUserStylesheet,
   shortSource,
+  ELEMENT_BACKED_PSEUDO_ELEMENTS,
   FILTER,
   STATUS,
 } = require("resource://devtools/shared/inspector/css-logic.js");
@@ -483,6 +484,12 @@ class CssLogic {
       if (domRule.selectorMatchesElement(idx, element)) {
         return true;
       }
+
+      for (const pseudoElement of ELEMENT_BACKED_PSEUDO_ELEMENTS) {
+        if (domRule.selectorMatchesElement(idx, element, pseudoElement)) {
+          return true;
+        }
+      }
     } while (
       // Loop on flattenedTreeParentNode instead of parentNode to reach the
       // shadow host from the shadow dom.
@@ -494,25 +501,34 @@ class CssLogic {
   }
 
   /**
-   * Check if the highlighted element or it's parents have matched selectors.
+   * Check if the highlighted element or its parents have matched selectors.
    *
-   * @param {Array} properties: The list of properties you want to check if they
+   * @param {Array<String>} properties: The list of properties you want to check if they
    * have matched selectors or not. For CSS variables, this will check if the variable
    * is set OR used in a matching rule.
-   * @return {object} An object that tells for each property if it has matched
-   * selectors or not. Object keys are property names and values are booleans.
+   * @return {Set<String>} A Set containing the properties that do have matched selectors.
    */
   hasMatchedSelectors(properties) {
     if (!this._matchedRules) {
       this._buildMatchedRules();
     }
 
-    const result = {};
+    const result = new Set();
 
-    this._matchedRules.some(function (value) {
-      const rule = value[0];
-      const status = value[1];
-      properties = properties.filter(property => {
+    for (const [rule, status] of this._matchedRules) {
+      // Getting the rule cssText can be costly, so cache it
+      let cssText;
+      const getCssText = () => {
+        if (cssText === undefined) {
+          cssText = rule.domRule.cssText;
+        }
+        return cssText;
+      };
+
+      // Loop through properties in reverse as we're removing items from it and we don't
+      // want to mess with the iteration.
+      for (let i = properties.length - 1; i >= 0; i--) {
+        const property = properties[i];
         // We just need to find if a rule has this property while it matches
         // the viewedElement (or its parents).
         if (
@@ -523,9 +539,7 @@ class CssLogic {
               // we may have false positive for dashed ident or the variable being
               // used in comment/string, but the tradeoff seems okay, as we would have
               // to parse the value of each declaration, which could be costly.
-              new RegExp(`${property}[^A-Za-z0-9_-]`).test(
-                rule.domRule.cssText
-              ))) &&
+              new RegExp(`${property}[^A-Za-z0-9_-]`).test(getCssText()))) &&
           (status == STATUS.MATCHED ||
             (status == STATUS.PARENT_MATCH &&
               InspectorUtils.isInheritedProperty(
@@ -533,14 +547,16 @@ class CssLogic {
                 property
               )))
         ) {
-          result[property] = true;
-          return false;
+          result.add(property);
+          // Once the property has a matched selector, we can remove it from the array
+          properties.splice(i, 1);
         }
-        // Keep the property for the next rule.
-        return true;
-      });
-      return !properties.length;
-    }, this);
+      }
+
+      if (!properties.length) {
+        return result;
+      }
+    }
 
     return result;
   }
@@ -577,7 +593,7 @@ class CssLogic {
         this.viewedElement === element ? STATUS.MATCHED : STATUS.PARENT_MATCH;
 
       try {
-        domRules = getCSSStyleRules(element);
+        domRules = getMatchingCSSRules(element);
       } catch (ex) {
         console.log("CL__buildMatchedRules error: " + ex);
         continue;
@@ -592,17 +608,13 @@ class CssLogic {
         this._matchedRules.push([rule, status, distance]);
       }
 
-      // getCSSStyleRules can return null with a shadow DOM element.
+      // getMatchingCSSRules can return null with a shadow DOM element.
       if (domRules !== null) {
-        // getCSSStyleRules returns ordered from least-specific to most-specific,
+        // getMatchingCSSRules returns ordered from least-specific to most-specific,
         // but we do want them from most-specific to least specific, so we need to loop
         // through the rules backward.
         for (let i = domRules.length - 1; i >= 0; i--) {
           const domRule = domRules[i];
-          if (!CSSStyleRule.isInstance(domRule)) {
-            continue;
-          }
-
           const sheet = this.getSheet(domRule.parentStyleSheet, -1);
           if (sheet._passId !== this._passId) {
             sheet.index = sheetIndex++;
@@ -1532,7 +1544,7 @@ class CssSelectorInfo {
 
   /**
    * Compare the current CssSelectorInfo instance to another instance.
-   * Since selectorInfos is computed from `InspectorUtils.getCSSStyleRules`,
+   * Since selectorInfos is computed from `InspectorUtils.getMatchingCSSRules`,
    * it's already sorted for regular cases. We only need to handle important values.
    *
    * @param  {CssSelectorInfo} that
