@@ -306,11 +306,13 @@ class Animation : public DOMEventTargetHelper,
   };
   // Note: we provide |aContext|/|aOtherContext| only when it is a cancelled
   // transition or animation (for overridding the target and animation index).
-  bool HasLowerCompositeOrderThan(
-      const Maybe<EventContext>& aContext, const Animation& aOther,
-      const Maybe<EventContext>& aOtherContext) const;
-  bool HasLowerCompositeOrderThan(const Animation& aOther) const {
-    return HasLowerCompositeOrderThan(Nothing(), aOther, Nothing());
+  int32_t CompareCompositeOrder(const Maybe<EventContext>& aContext,
+                                const Animation& aOther,
+                                const Maybe<EventContext>& aOtherContext,
+                                nsContentUtils::NodeIndexCache&) const;
+  int32_t CompareCompositeOrder(const Animation& aOther,
+                                nsContentUtils::NodeIndexCache& aCache) const {
+    return CompareCompositeOrder(Nothing(), aOther, Nothing(), aCache);
   }
 
   /**
@@ -341,8 +343,10 @@ class Animation : public DOMEventTargetHelper,
    * Any properties contained in |aPropertiesToSkip| will not be added or
    * updated in |aComposeResult|.
    */
-  void ComposeStyle(StyleAnimationValueMap& aComposeResult,
-                    const InvertibleAnimatedPropertyIDSet& aPropertiesToSkip);
+  void ComposeStyle(
+      StyleAnimationValueMap& aComposeResult,
+      const InvertibleAnimatedPropertyIDSet& aPropertiesToSkip,
+      EndpointBehavior aEndpointBehavior = EndpointBehavior::Exclusive);
 
   void NotifyEffectTimingUpdated();
   void NotifyEffectPropertiesUpdated();
@@ -358,8 +362,6 @@ class Animation : public DOMEventTargetHelper,
    */
   virtual void MaybeQueueCancelEvent(const StickyTimeDuration& aActiveTime) {};
 
-  Maybe<uint32_t>& CachedChildIndexRef() { return mCachedChildIndex; }
-
   void SetPartialPrerendered(uint64_t aIdOnCompositor) {
     mIdOnCompositor = aIdOnCompositor;
     mIsPartialPrerendered = true;
@@ -549,10 +551,6 @@ class Animation : public DOMEventTargetHelper,
   // possible for two different objects to have the same index.
   uint64_t mAnimationIndex;
 
-  // While ordering Animation objects for event dispatch, the index of the
-  // target node in its parent may be cached in mCachedChildIndex.
-  Maybe<uint32_t> mCachedChildIndex;
-
   // Indicates if the animation is in the pending state (and what state it is
   // waiting to enter when it finished pending).
   enum class PendingState : uint8_t { NotPending, PlayPending, PausePending };
